{* ************************************************************************* *}
{   / ini- LCard_AD.ini                    }
{* ************************************************************************* *}
unit Ini;
{$IFNDEF DEBUG}
{$A+,B-,D-,F-,G+,I-,K+,L-,N+,P+,Q-,R-,S-,T+,V-,W-,X+,Y-}
{$ENDIF}

interface

procedure ReadIniSettings;
procedure WriteIniSettings;

implementation
uses
    LZExpand, WinTypes, WinProcs, Classes, SysUtils, IniFiles, LUtil, GlobVar, LCard_AD;

const
    General    ='General';
    SlotName   ='Slot_';
    IniFileName='LCARD_AD.INI';

{* ************************************************************************* *}
procedure ReadIniSettings;
var
    i:word;
    s:string;
    p:string;
    LbiosIni:TIniFile;
    BIOSPath:TStringList;

Begin
    {      }
    LbiosIni:=TIniFile.Create(IniFileName);

    with LbiosIni do
    Begin
        {    DLL- }
        InitStringToWord(ReadString(General, 'MaxSlot', '2'), MaxSlot, 2);
        if MaxSlot>MaxSlotArray then MaxSlot:=2;
        Language:=ReadBool(General, 'Language', True);
        IntEnabled:=ReadBool(General, 'IntEnabled', True);
        IgnoreError:=ReadBool(General, 'IgnoreError', False);
        NeedLog:=ReadBool(General, 'NeedLog', False);
        NeedLogIO:=ReadBool(General, 'NeedLogIO', False);

        BIOSPath:=TStringList.Create;

        with BIOSPath do
        Begin
            Sorted:=True;
            Duplicates:=dupIgnore;
            Add(ExtractFilePath(ParamStr(0)));

            for i:=FirstBoard to LastBoard do
            Begin
                p:=ReadString('Default_BIOS', BoardNames[i], '');
                DefaultBiosName[i]:=p;
                if p<>'' then
                Begin
                    p:=ExtractFilePath(p);
                    if p<>'' then Add(p);
                End;
            End;

            {     }
            for i:=0 to MaxSlot do
            Begin
                with SlotArray[i] do
                Begin
                    s:=SlotName+IntToStr(i);

                    InitStringToWord(ReadString(s, 'BoardType', '0'), BoardType, 0);
                    if BoardType>LastBoard then BoardType:=Unknown;
                    InitStringToLongint('$'+ReadString(s, 'BaseAddress', '378'), BoardBase, $378);
                    InitStringToWord(ReadString(s, 'IRQ', '7'), BoardIrq, 7);
                    BoardIrq:=BoardIrq and $F;
                    InitStringToWord(ReadString(s, 'BoardInterface', '0'), BoardInterface, EPP_INTERFACE);
                    if BoardInterface>LastInterface then BoardInterface:=EPP_INTERFACE;
                    InitStringToWord(ReadString(s, 'Binding', '0'), Binding, 0);
                    if Binding>MaxSlot then Binding:=0;
                    InitStringToWord(ReadString(s, 'CraitAddress', '0'), CraitAddress, 0);
                    if CraitAddress>63 then CraitAddress:=0;
                    InitStringToWord(ReadString(s, 'DSPType', '0'), DspType, 0);
                    if DspType>1 then DspType:=0;
                    InitStringToWord(ReadString(s, 'WaitState', '0'), WaitState, 0);
                    InitStringToWord(ReadString(s, 'InterDelay', '24'), InterDelay, 24);
                    p:=ReadString(s, 'BIOS_Name', '');
                    AutoLoadBIOS   :=ReadBool(s, 'BIOS_Autoload', True);
                    NoDSL          :=ReadBool(s, 'BIOS_No_DSL', False);
                    Bios_IsOldStyle:=ReadBool(s, 'BIOS_IsOldStyle', True);

                    if p<>'' then
                    Begin
                        if FileExists(p) then BiosName:=p else BiosName:=ChangeFileExt(DefaultBiosName[BoardType], '');
                        p:=ExtractFilePath(p);
                        if p<>'' then Add(p);
                    End;
                End;
            End;

            p:='';
            for i:=0 to Count-1 do
            Begin
                if Length(p)+Length(Strings[i])<SizeOf(s) then p:=p+Strings[i]+';';
            End;
            Dec(p[0]);

            Free;

            for i:=FirstBoard to LastBoard do
            Begin
                if i in ADSPBoards then
                Begin
                    if DefaultBiosName[i]='' then
                    Begin
                        s:=FileSearch(BiosNames[i], p);
                        if s<>'' then DefaultBiosName[i]:=ExpandFileName(s);
                    End;
                End;
            End;

            for i:=0 to MaxSlot do
            Begin
                with SlotArray[i] do
                Begin
                    if BiosName='' then BiosName:=ChangeFileExt(DefaultBiosName[BoardType], '');
                End;
            End;
        End;

        Free;
    End;
End;

{* ************************************************************************* *}
procedure WriteIniSettings;
var
    i:word;
    s:string;
    LbiosIni:TIniFile;

Begin
    LbiosIni:=TIniFile.Create(IniFileName);

    with LbiosIni do
    Begin
        {   }
        WriteInteger(General, 'MaxSlot', MaxSlot);
        WriteBool(General, 'Language', Language);
        WriteBool(General, 'IntEnabled', IntEnabled);
        WriteBool(General, 'IgnoreError', IgnoreError);
        WriteBool(General, 'NeedLog', NeedLog);
        WriteBool(General, 'NeedLogIO', NeedLogIO);

        {   }
        for i:=0 to MaxSlot do
        Begin
            with SlotArray[i] do
            Begin
                s:=SlotName+IntToStr(i);

                WriteInteger(s, 'BoardType', BoardType);
                WriteString(s, 'BaseAddress', IntToHex(BoardBase,0));
                WriteInteger(s, 'IRQ', BoardIrq);
                WriteInteger(s, 'BoardInterface', BoardInterface);
                WriteInteger(s, 'Binding', Binding);
                WriteInteger(s, 'CraitAddress', CraitAddress);
                WriteInteger(s, 'DSPType', DspType);
                WriteInteger(s, 'WaitState', WaitState);
                WriteInteger(s, 'InterDelay', InterDelay);
                WriteString(s, 'BIOS_Name', BiosName);
                WriteBool(s, 'BIOS_Autoload', AutoLoadBIOS);
                WriteBool(s, 'BIOS_No_DSL', NoDSL);
                WriteBool(s, 'BIOS_IsOldStyle', Bios_IsOldStyle);
            End;
        End;

        {   BIOS     }
        for i:=FirstBoard to LastBoard do
        Begin
            if i in ADSPBoards then
            Begin
                WriteString('Default_BIOS', BoardNames[i], DefaultBiosName[i]);
            End;
        End;
   End;

   LbiosIni.Free;
End;

End.
