{* ************************************************************************* *}
{     (- )                                      }
{        !!!                                                          }
{   ,   ,                   }
{    : FIXED PRELOAD PERMANENT         }
{    MOVEABLE DEMANDLOAD DISCARDABLE           }
{ !!!                                                                   }
{* ************************************************************************* *}
unit TestDlg;
{$C FIXED PRELOAD PERMANENT }
{$IFNDEF DEBUG}
{$A+,B-,D-,F-,G+,I-,K+,L-,N+,P+,Q-,R-,S-,T+,V-,W-,X+,Y-}
{$ENDIF}

interface
uses
    WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
    StdCtrls, ExtCtrls, Spin;

type
  TTestBoardForm = class(TForm)
    TestPanel: TPanel;
    ShowPanel: TPanel;
    ShowPaintBox: TPaintBox;
    ChanE: TEdit;
    ChanSB: TSpinButton;
    ChanLab: TLabel;
    RateE: TEdit;
    RateSB: TSpinButton;
    RateLab: TLabel;
    OkBtn: TBitBtn;
    InputModeRG: TRadioGroup;
    RunTimer: TTimer;

    procedure FormActivate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure ChanSBDownClick(Sender: TObject);
    procedure ChanSBUpClick(Sender: TObject);
    procedure RateSBDownClick(Sender: TObject);
    procedure RateSBUpClick(Sender: TObject);
    procedure InputModeRGClick(Sender: TObject);
    procedure RunTimerTimer(Sender: TObject);
  private
    { Private declarations }
  public
    procedure PlotGraph;
    procedure GetCtrlVals;
    procedure PutVariables;
    procedure SetLanguage;
    { Public declarations }
  end;

var
  TestBoardForm: TTestBoardForm;

procedure Handler;interrupt;
function  MyErrorHandler(Error:longint; Info:longint):integer;export;

implementation
uses
    SysUtils, LBios, LUtil, GlobVar, LCard_AD;
{$R *.DFM}

const
    NKadr       =400;
    TIMER_INPUT =0;
    KADR_INPUT  =1;
    IRQ_INPUT   =2;
    MEMORY_INPUT=3;
    DMA_INPUT   =4;

    ModeNames:array[0..4] of string[30]=
        ('Timer synchro.',
         'Kadr input',
         'IRQ input',
         'Internal Memory Input',
         'DMA input'
        );

type
    TData=array [0..NKadr-1] of integer;

var
    Chan         :word;  {    }
    Rate         :word;
    InputMode    :word;
    pData        :^TData;
    NChannel     :word;
    NKadrAcquired:word;
    First_DM_Page:word;
    MemState     :word;
    AllowedModes :array [0..9] of byte;
    InputStarted :boolean;
    BeginTime    :longint;

{* ************************************************************************* *}
{                                     }
{* ************************************************************************* *}
procedure StartInput;
var
    i:word;
    Channels:array [0..31] of word;

Begin
    ErrorOccurred:=False;
    BeginTime:=GetTickCount;

    for i:=0 to NChannel-1 do Channels[i]:=CREATE_CHANNEL((Chan+i) mod 16, 0, 0);

    case InputMode of

        TIMER_INPUT: SOFT(pData^, NKadr, Channels, NChannel, Rate);

        KADR_INPUT:  for i:=0 to NKadr-1 do KADR(pData^[i*NChannel], Channels, NChannel);

        IRQ_INPUT:
        Begin
            NKadrAcquired:=0;
            case ThisSlot.BoardType of
                E330:
                Begin
                    SET_SOFT_PARM_E330(NChannel, Channels);
                    SET_TIME_PARAMETERS_E330(Rate);
                    IRQ_SLOW_START_E330(1, @Handler);
                End;
                L1221:
                Begin
                    PUT_DM_WORD_L1221($3D51, $0001);
                    PUT_DM_WORD_L1221($3D53, 0);
                    PUT_DM_WORD_L1221($3D54, 0);

                    GET_DM_WORD_L1221(0);
                    INIT_INTR(@Handler);
                    PUT_DM_WORD_L1221($3D4F, 1);
                    COMMAND_L1221(5);
                End;
            else
                Begin
                    SOFT_INTR(Rate, @Handler, Chan, NChannel);
                End;
            End;
        End;

        {     }
        MEMORY_INPUT: SOFT_MEMORY_START(NKadr, Channels, NChannel, Rate, First_DM_Page, 0, 0);

        DMA_INPUT: DMAALL(pData^, NKadr, Channels, Nchannel, Rate, 0);
    End;
End;

{* ************************************************************************* *}
function TestInputDone:Boolean;
Begin
    case InputMode of
        TIMER_INPUT,
        KADR_INPUT:   TestInputDone:=True;
        IRQ_INPUT:    TestInputDone:=NKadrAcquired>=NKadr;
        MEMORY_INPUT: TestInputDone:=TEST_FOR_INPUT_END<>0;
        DMA_INPUT:    TestInputDone:=DMA_TEST<>0;
    End;
End;

{* ************************************************************************* *}
function FinishInput:boolean;
Begin
    case InputMode of

        IRQ_INPUT:
        Begin
            if ThisSlot.BoardType=L1221 then
            Begin
                PUT_DM_WORD_L1221($3D4F, 0);
                COMMAND_L1221(5);
            End;
            STOP_INTR;
        End;

        MEMORY_INPUT:
        Begin
            GET_INPUT_DATA(pData^, NKadr*NChannel);
            if ThisSlot.BoardType=E330 then STOP_FUNC_E330;
        End;

        DMA_INPUT: DMA_OFF;
    End;
End;

{* ************************************************************************* *}
{     
{* ************************************************************************* *}
procedure SetAllowedModes;
var
    i:integer;

procedure AddOneMode(Mode:Integer);
Begin
    AllowedModes[i]:=Mode;
    Inc(i);
    TestBoardForm.InputModeRG.Items.Add(ModeNames[Mode]);
End;

Begin
    {      IRQ }
    TestBoardForm.InputModeRG.Items.Clear;
    i:=0;

    case ThisSlot.BoardType of
        L1221: AddOneMode(IRQ_INPUT);
        L750:
        Begin
            AddOneMode(TIMER_INPUT);
            AddOneMode(KADR_INPUT);
        End;
    else
        Begin
            AddOneMode(TIMER_INPUT);
            AddOneMode(KADR_INPUT);
            if ThisSlot.BoardType in ADSPBoards then AddOneMode(IRQ_INPUT);
            if (MemState<>0) then AddOneMode(MEMORY_INPUT);
            if MemState=1 then First_DM_Page:=0;
            if ThisSlot.BoardType in (ADSPBoards-[E330])then AddOneMode(DMA_INPUT);
        End;
    End;
    TestBoardForm.InputModeRG.ItemIndex:=0;
End;

{* ************************************************************************* *}
procedure TTestBoardForm.GetCtrlVals;
Begin
    Chan:=StrToInt(ChanE.Text);
    Rate:=StrToInt(RateE.Text);
    if Chan<0 then Chan:=0;
    if Chan>31 then Chan:=31;
    if Rate<10 then Rate:=10;
    InputMode:=AllowedModes[InputModeRG.ItemIndex];
End;

{* ************************************************************************* *}
procedure TTestBoardForm.PlotGraph;
var
    i          :word;
    j          :integer;
    X0,Y0,X1,Y1:integer;
    Max        :integer;

Begin
    case ThisSlot.BoardType of
        L750,
        L1211: for i:=0 to NKadr*NChannel-1 do pData^[i]:=((pData^[i] and $FFF)-2048);
    End;

    with ShowPaintBox do
    Begin
        X0:=0;
        X1:=BoundsRect.Right-BoundsRect.Left;
        Y0:=0;
        Y1:=BoundsRect.Bottom-BoundsRect.Top;
        Canvas.Brush.Color:=clNavy;
        Canvas.Rectangle(0, 0, X1, Y1);
        Canvas.Pen.Color:=clYellow;
        Canvas.MoveTo(0, Y1 div 2);
        Canvas.LineTo(X1, Y1 div 2);

        {   }
        Max:=1;
        for j:=0 to NKadr*NChannel-1 do if Abs(pData^[j])>Max then Max:=Abs(pData^[j]);

        Canvas.MoveTo(Round(1/NKadr*X1), -Round(pData^[1]/Max*Y1/2*0.9)+Y1 div 2);
        { sic!   2-   }
        for j:=1 to NKadr-1 do Canvas.LineTo(Round(j/NKadr*X1), -Round(pData^[j]/Max*Y1/2*0.9)+Y1 div 2);
    End;
end;

{* ************************************************************************* *}
procedure TTestBoardForm.SetLanguage;
Begin
    TestBoardForm.Caption  :=MyLoadStr(TestBoardFormCaption);
    ChanSB.Hint            :=MyLoadStr(ChanSBHint);
    ChanLab.Caption        :=MyLoadStr(ChanLabCaption);
    ChanE.Hint             :=MyLoadStr(ChanEHint);
    TestPanel.Caption      :=MyLoadStr(TestPanelCaption);
    RateSB.Hint            :=MyLoadStr(RateSBHint);
    RateLab.Caption        :=MyLoadStr(RateLabCaption);
    RateE.Hint             :=MyLoadStr(RateEHint);
    InputModeRG.Caption    :=MyLoadStr(InputModeRGCaption);
    InputModeRG.Hint       :=MyLoadStr(InputModeRGHint);
End;

{* ************************************************************************* *}
procedure TTestBoardForm.ChanSBDownClick(Sender: TObject);
begin
    GetCtrlVals;
    Dec(Chan);
    if Chan<0 then Chan:=0;
    PutVariables;
    PlotGraph;
end;

{* ************************************************************************* *}
procedure TTestBoardForm.ChanSBUpClick(Sender: TObject);
begin
    GetCtrlVals;
    Inc(Chan);
    PutVariables;
    PlotGraph;
end;

{* ************************************************************************* *}
procedure TTestBoardForm.RateSBDownClick(Sender: TObject);
begin
   GetCtrlVals;
   Rate:=Rate-10;
   if Rate<10 then Rate:=10;
   PutVariables;
   PlotGraph;
end;

{* ************************************************************************* *}
procedure TTestBoardForm.RateSBUpClick(Sender: TObject);
begin
   GetCtrlVals;
   Rate:=Rate+10;
   PutVariables;
   PlotGraph;
end;

{* ************************************************************************* *}
{     BIOS  ,    -> !}
{* ************************************************************************* *}
procedure TTestBoardForm.FormActivate(Sender: TObject);
Begin
    MemState:=MEMORY_PM_STATE;
    MemState:=MEMORY_STATE;
    SetAllowedModes;

    New(pData);
    GetCtrlVals;
    NChannel:=1;
    PutVariables;

    SetLanguage;

    InputStarted:=False;
    if ThisSlot.BoardType=L750 then SET_REFERENCE_L750(True);
    RunTimer.Enabled:=True;
End;

{* ************************************************************************* *}
procedure TTestBoardForm.FormDeactivate(Sender: TObject);
begin
   RunTimer.Enabled:=False;
   InputModeRG.ItemIndex:=0;
   if InputStarted then FinishInput;
   Dispose(pData);
end;

{* ************************************************************************* *}
procedure TTestBoardForm.InputModeRGClick(Sender: TObject);
begin
    if InputStarted then FinishInput;
    InputStarted:=False;
end;

{* ************************************************************************* *}
procedure TTestBoardForm.PutVariables;
begin
    ChanE.Text:=IntToStr(Chan);
    RateE.Text:=IntToStr(Rate);
end;

{* ************************************************************************* *}
procedure TTestBoardForm.RunTimerTimer(Sender: TObject);
Begin
    if InputStarted then
    Begin
        { ,   .  ,       }
        if TestInputDone then
        Begin
            FinishInput;
            InputStarted:=False;
            if ErrorOccurred then Close else PlotGraph;
        End else
        Begin
            if GetTickCount-BeginTime>1000 then
            Begin
                FinishInput;
                InputStarted:=False;
                if ThisSlot.BoardType=L750 then
                Begin
                    if StrToInt(RateE.Text)<10000 then RateE.Text:=IntToStr(Rate shr 1) else RateE.Text:='1000';
                End else
                Begin
                    if StrToInt(RateE.Text)<10000 then RateE.Text:=IntToStr(Rate*2) else RateE.Text:='1000';
                End;
                Exit;
            End;
        End;
    End else
    Begin
        GetCtrlVals;
        StartInput;
        InputStarted:=True;
    End;
End;

{* ************************************************************************* *}
{$Q-,R-,S-,I-}
procedure Handler;
var
    i:word;

Begin
    {      ->   }
    if ThisSlot.BoardType=L1221 then
    Begin
        GET_DATA_MEMORY_L1221(pData^, NKadr, 0);
        NKadrAcquired:=NKadr;
    End else
    Begin
        if NKadrAcquired<NKadr then
        Begin
            for i:=0 to NChannel-1 do pData^[NKadrAcquired*NChannel+i]:=READ_DATA;
            Inc(NKadrAcquired);
        End else
        Begin
            for i:=0 to NChannel-1 do READ_DATA;
        End;
    End;
    RESET_IRQ;
End;

{* ************************************************************************* *}
function MyErrorHandler(Error:longint; Info:longint):integer;
Begin
    if Error=$20002002 then
    Begin
        ErrorOccurred:=True;
        MyErrorHandler:=IDCANCEL;
    End else
    Begin
        ErrorOccurred:=True;
        MyErrorHandler:=102;
    End;
End;

End.

